@@ -43,6 +43,7 @@ extern gboolean GdmXdmcp;
 extern gint xdmcp_sessions;
 extern gint flexi_servers;
 extern gint xdmcp_pending;
+extern gboolean gdm_in_final_cleanup;
 extern GSList *displays;
 extern GdmConnection *fifoconn;
 extern GdmConnection *pipeconn;
@@ -161,6 +162,64 @@ gdm_display_check_loop (GdmDisplay *disp)
   return TRUE;
 }
 
+static void
+whack_old_slave (GdmDisplay *d)
+{
+    time_t t = time (NULL);
+    gboolean waitsleep = TRUE;
+    /* if we have DISPLAY_DEAD set, then this has already been killed */
+    if (d->dispstat == DISPLAY_DEAD)
+	    waitsleep = FALSE;
+    /* Kill slave */
+    if (d->slavepid > 1 &&
+	(d->dispstat == DISPLAY_DEAD || kill (d->slavepid, SIGTERM) == 0)) {
+	    int exitstatus;
+	    int ret;
+wait_again:
+		
+	    if (waitsleep)
+		    /* wait for some signal, yes this is a race */
+		    sleep (10);
+	    waitsleep = TRUE;
+	    errno = 0;
+	    ret = waitpid (d->slavepid, &exitstatus, WNOHANG);
+	    if (ret <= 0) {
+		    /* rekill the slave to tell it to
+		       hurry up and die if we're getting
+		       killed ourselves */
+		    if (ve_signal_was_notified (SIGTERM) ||
+			ve_signal_was_notified (SIGINT) ||
+			ve_signal_was_notified (SIGHUP) ||
+			t + 10 <= time (NULL)) {
+			    gdm_debug ("whack_old_slave: GOT ANOTHER SIGTERM (or it was 10 secs already), killing slave again");
+			    t = time (NULL);
+			    kill (d->slavepid, SIGTERM);
+			    goto wait_again;
+		    } else if (ret < 0 && errno == EINTR) {
+			    goto wait_again;
+		    }
+	    }
+
+	    if (WIFSIGNALED (exitstatus)) {
+		    gdm_debug ("whack_old_slave: Slave crashed (signal %d), killing its children",
+			       (int)WTERMSIG (exitstatus));
+
+		    if (d->sesspid > 1)
+			    kill (-(d->sesspid), SIGTERM);
+		    d->sesspid = 0;
+		    if (d->greetpid > 1)
+			    kill (-(d->greetpid), SIGTERM);
+		    d->greetpid = 0;
+		    if (d->chooserpid > 1)
+			    kill (-(d->chooserpid), SIGTERM);
+		    d->chooserpid = 0;
+		    if (d->servpid > 1)
+			    kill (d->servpid, SIGTERM);
+		    d->servpid = 0;
+	    }
+    }
+    d->slavepid = 0;
+}
 
 /**
  * gdm_display_manage:
@@ -191,21 +250,14 @@ gdm_display_manage (GdmDisplay *d)
 	    gdm_debug ("gdm_display_manage: Old slave pid is %d", (int)d->slavepid);
 
     /* If we have an old slave process hanging around, kill it */
-    gdm_sigchld_block_push ();
-    if (d->slavepid > 1 &&
-	kill (d->slavepid, SIGTERM) == 0)
-	    ve_waitpid_no_signal (d->slavepid, 0, 0);
-    d->slavepid = 0;
-    gdm_sigchld_block_pop ();
+    /* This shouldn't be a normal code path however, so it doesn't matter
+     * that we are hanging */
+    whack_old_slave (d);
 
     d->managetime = time (NULL);
 
     /* Fork slave process */
-    gdm_sigchld_block_push ();
-    gdm_sigterm_block_push ();
     pid = d->slavepid = fork ();
-    gdm_sigterm_block_push ();
-    gdm_sigchld_block_pop ();
 
     switch (pid) {
 
@@ -249,28 +301,12 @@ gdm_display_manage (GdmDisplay *d)
 
 	d->slave_notify_fd = fds[0];
 
-	if (d->type == TYPE_LOCAL) {
-	    gdm_slave_start (d);
-	    /* if we ever return, bad things are happening */
-	    gdm_server_stop (d);
-	    _exit (DISPLAY_ABORT);
-	} else if (SERVER_IS_FLEXI (d)) {
-	    gdm_slave_start (d);
-	    gdm_server_stop (d);
-	    /* we expect to return after the session finishes */
-	    _exit (DISPLAY_REMANAGE);
-	} else if (d->type == TYPE_XDMCP &&
-		   d->dispstat == XDMCP_MANAGED) {
-	    gdm_slave_start (d);
-	    gdm_server_stop (d);
-	    /* we expect to return after the session finishes */
-	    _exit (DISPLAY_REMANAGE);
-	}
+	gdm_slave_start (d);
+	/* should never retern */
 
-	/* yaikes, how did we ever get here, though I suppose
-	 * it could be possible if XMDCP thing wasn't really set up */
+	/* yaikes, how did we ever get here? */
 	gdm_server_stop (d);
-	_exit (DISPLAY_ABORT);
+	_exit (DISPLAY_REMANAGE);
 
 	break;
 
@@ -286,7 +322,7 @@ gdm_display_manage (GdmDisplay *d)
 	gdm_debug ("gdm_display_manage: Forked slave: %d",
 		   (int)pid);
 	d->master_notify_fd = fds[1];
-	close (fds[0]);
+	IGNORE_EINTR (close (fds[0]));
 	break;
     }
 
@@ -326,18 +362,15 @@ gdm_display_unmanage (GdmDisplay *d)
     gdm_debug ("gdm_display_unmanage: Stopping %s (slave pid: %d)",
 	       d->name, (int)d->slavepid);
 
-    /* Kill slave */
-    gdm_sigchld_block_push ();
-    if (d->slavepid > 1 &&
-	kill (d->slavepid, SIGTERM) == 0)
-	    ve_waitpid_no_signal (d->slavepid, 0, 0);
-    d->slavepid = 0;
-    gdm_sigchld_block_pop ();
+    /* Kill slave, this may in fact hang for a bit at least until the
+     * slave dies, which should be ASAP though */
+    whack_old_slave (d);
     
-    if (d->type == TYPE_LOCAL)
-	d->dispstat = DISPLAY_DEAD;
-    else /* TYPE_XDMCP,TYPE_FLEXI,TYPE_FLEXI_XNEST */
+    d->dispstat = DISPLAY_DEAD;
+    if (d->type != TYPE_LOCAL)
 	gdm_display_dispose (d);
+
+    gdm_debug ("gdm_display_unmanage: Display stopped");
 }
 
 
@@ -361,12 +394,12 @@ gdm_display_dispose (GdmDisplay *d)
     }
 
     if (d->slave_notify_fd >= 0) {
-	    close (d->slave_notify_fd);
+	    IGNORE_EINTR (close (d->slave_notify_fd));
 	    d->slave_notify_fd = -1;
     }
 
     if (d->master_notify_fd >= 0) {
-	    close (d->master_notify_fd);
+	    IGNORE_EINTR (close (d->master_notify_fd));
 	    d->master_notify_fd = -1;
     }
 
@@ -407,7 +440,7 @@ gdm_display_dispose (GdmDisplay *d)
     d->authfile_gdm = NULL;
 
     if (d->xnest_temp_auth_file != NULL)
-	    unlink (d->xnest_temp_auth_file);
+	    IGNORE_EINTR (unlink (d->xnest_temp_auth_file));
     g_free (d->xnest_temp_auth_file);
     d->xnest_temp_auth_file = NULL;
 
